@@ -1,9 +1,24 @@
-/* -*- c-file-style: "linux" -*-
-
-   Weiss 1/1999
-   Batch utilities for rsync.
-
-*/
+/*
+ * Support for the batch-file options.
+ *
+ * Copyright (C) 1999 Weiss
+ * Copyright (C) 2004 Chris Shoemaker
+ * Copyright (C) 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "rsync.h"
 #include "zlib/zlib.h"
