@@ -1,7 +1,9 @@
-/* -*- c-file-style: "linux"; -*-
+/*
+ * The socket based protocol for setting up a connection with rsyncd.
  *
- * Copyright (C) 1998-2001 by Andrew Tridgell <tridge@samba.org>
- * Copyright (C) 2001-2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 1998-2001 Andrew Tridgell <tridge@samba.org>
+ * Copyright (C) 2001-2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,22 +15,16 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/**
- * @file
- *
- * The socket based protocol for setting up a connection with
- * rsyncd.
- **/
-
 #include "rsync.h"
 
 extern int verbose;
 extern int quiet;
+extern int output_motd;
 extern int list_only;
 extern int am_sender;
 extern int am_server;
@@ -44,10 +40,13 @@ extern int protocol_version;
 extern int io_timeout;
 extern int no_detach;
 extern int default_af_hint;
+extern int logfile_format_has_i;
+extern int logfile_format_has_o_or_i;
 extern mode_t orig_umask;
 extern char *bind_address;
 extern char *sockopts;
 extern char *config_file;
+extern char *logfile_format;
 extern char *files_from;
 extern char *tmpdir;
 extern struct chmod_mode_struct *chmod_modes;
@@ -55,8 +54,6 @@ extern struct filter_list_struct server_filter_list;
 
 char *auth_user;
 int read_only = 0;
-int daemon_log_format_has_i = 0;
-int daemon_log_format_has_o_or_i = 0;
 int module_id = -1;
 struct chmod_mode_struct *daemon_chmod_modes;
 
@@ -113,7 +110,7 @@ int start_socket_client(char *host, char *path, int argc, char *argv[])
 	return ret ? ret : client_run(fd, fd, -1, argc, argv);
 }
 
-int start_inband_exchange(char *user, char *path, int f_in, int f_out, 
+int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 			  int argc)
 {
 	int i;
@@ -209,7 +206,10 @@ int start_inband_exchange(char *user, char *path, int f_in, int f_out,
 			return -1;
 		}
 
-		rprintf(FINFO, "%s\n", line);
+		/* This might be a MOTD line or a module listing, but there is
+		 * no way to differentiate it.  The manpage mentions this. */
+		if (output_motd)
+			rprintf(FINFO, "%s\n", line);
 	}
 	kluge_around_eof = 0;
 
@@ -328,13 +328,12 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 	if (lp_read_only(i))
 		read_only = 1;
 
-	if (lp_transfer_logging(i)) {
-		if (log_format_has(lp_log_format(i), 'i'))
-			daemon_log_format_has_i = 1;
-		if (daemon_log_format_has_i
-		    || log_format_has(lp_log_format(i), 'o'))
-			daemon_log_format_has_o_or_i = 1;
-	}
+	if (lp_transfer_logging(i) && !logfile_format)
+		logfile_format = lp_log_format(i);
+	if (log_format_has(logfile_format, 'i'))
+		logfile_format_has_i = 1;
+	if (logfile_format_has_i || log_format_has(logfile_format, 'o'))
+		logfile_format_has_o_or_i = 1;
 
 	am_root = (MY_UID() == 0);
 
@@ -394,7 +393,7 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 	parse_rule(&server_filter_list, p, MATCHFLG_WORD_SPLIT,
 		   XFLG_ABS_IF_SLASH | XFLG_OLD_PREFIXES);
 
-	log_init();
+	log_init(1);
 
 #ifdef HAVE_PUTENV
 	if (*lp_prexfer_exec(i) || *lp_postxfer_exec(i)) {
@@ -423,17 +422,18 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 				return -1;
 			}
 			if (pid) {
-				char *ret1, *ret2;
+				if (asprintf(&p, "RSYNC_PID=%ld", (long)pid) > 0)
+					putenv(p);
 				if (wait_process(pid, &status, 0) < 0)
 					status = -1;
-				if (asprintf(&ret1, "RSYNC_RAW_STATUS=%d", status) > 0)
-					putenv(ret1);
+				if (asprintf(&p, "RSYNC_RAW_STATUS=%d", status) > 0)
+					putenv(p);
 				if (WIFEXITED(status))
 					status = WEXITSTATUS(status);
 				else
 					status = -1;
-				if (asprintf(&ret2, "RSYNC_EXIT_STATUS=%d", status) > 0)
-					putenv(ret2);
+				if (asprintf(&p, "RSYNC_EXIT_STATUS=%d", status) > 0)
+					putenv(p);
 				system(lp_postxfer_exec(i));
 				_exit(status);
 			}
@@ -443,6 +443,8 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 		 * send us the user's request via a pipe. */
 		if (*lp_prexfer_exec(i)) {
 			int fds[2];
+			if (asprintf(&p, "RSYNC_PID=%ld", (long)getpid()) > 0)
+				putenv(p);
 			if (pipe(fds) < 0 || (pre_exec_pid = fork()) < 0) {
 				rsyserr(FLOG, errno, "pre-xfer exec preparation failed");
 				io_printf(f_out, "@ERROR: pre-xfer exec preparation failed\n");
@@ -456,9 +458,8 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 				len = read_arg_from_pipe(fds[0], buf, BIGPATHBUFLEN);
 				if (len <= 0)
 					_exit(1);
-				if (asprintf(&p, "RSYNC_REQUEST=%s", buf) < 0)
-					out_of_memory("rsync_module");
-				putenv(p);
+				if (asprintf(&p, "RSYNC_REQUEST=%s", buf) > 0)
+					putenv(p);
 				for (j = 0; ; j++) {
 					len = read_arg_from_pipe(fds[0], buf,
 								 BIGPATHBUFLEN);
@@ -467,9 +468,8 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 							break;
 						_exit(1);
 					}
-					if (asprintf(&p, "RSYNC_ARG%d=%s", j, buf) < 0)
-						out_of_memory("rsync_module");
-					putenv(p);
+					if (asprintf(&p, "RSYNC_ARG%d=%s", j, buf) > 0)
+						putenv(p);
 				}
 				close(fds[0]);
 				close(STDIN_FILENO);
@@ -507,7 +507,7 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 			return -1;
 		}
 
-		if (!push_dir("/")) {
+		if (!push_dir("/", 0)) {
 			rsyserr(FLOG, errno, "chdir %s failed\n",
 				lp_path(i));
 			io_printf(f_out, "@ERROR: chdir failed\n");
@@ -515,7 +515,7 @@ static int rsync_module(int f_in, int f_out, int i, char *addr, char *host)
 		}
 
 	} else {
-		if (!push_dir(lp_path(i))) {
+		if (!push_dir(lp_path(i), 0)) {
 			rsyserr(FLOG, errno, "chdir %s failed\n",
 				lp_path(i));
 			io_printf(f_out, "@ERROR: chdir failed\n");
@@ -841,7 +841,7 @@ int daemon_main(void)
 	if (bind_address == NULL && *lp_bind_address())
 		bind_address = lp_bind_address();
 
-	log_init();
+	log_init(0);
 
 	rprintf(FLOG, "rsyncd version %s starting, listening on port %d\n",
 		RSYNC_VERSION, rsync_port);
