@@ -1,38 +1,37 @@
-/* -*- c-file-style: "linux" -*-
-
-   rsync -- fast file replication program
-
-   Copyright (C) 1996-2000 by Andrew Tridgell
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2002 by Martin Pool <mbp@samba.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+/*
+ * Routines that are exclusive to the generator process.
+ *
+ * Copyright (C) 1996-2000 Andrew Tridgell
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "rsync.h"
 
 extern int verbose;
 extern int dry_run;
 extern int do_xfers;
-extern int log_format_has_i;
-extern int daemon_log_format_has_i;
+extern int stdout_format_has_i;
+extern int logfile_format_has_i;
 extern int am_root;
 extern int am_server;
 extern int am_daemon;
 extern int do_progress;
-extern int recurse;
 extern int relative_paths;
 extern int implied_dirs;
 extern int keep_dirlinks;
@@ -51,7 +50,7 @@ extern int delete_during;
 extern int delete_after;
 extern int module_id;
 extern int ignore_errors;
-extern int remove_sent_files;
+extern int remove_source_files;
 extern int delay_updates;
 extern int update_only;
 extern int ignore_existing;
@@ -79,6 +78,7 @@ extern int copy_dest;
 extern int link_dest;
 extern int whole_file;
 extern int list_only;
+extern int new_root_dir;
 extern int read_batch;
 extern int safe_symlinks;
 extern long block_size; /* "long" because popt can't set an int32. */
@@ -291,8 +291,8 @@ static void do_delete_pass(struct file_list *flist)
 	STRUCT_STAT st;
 	int j;
 
-	if (dry_run > 1 /* destination doesn't exist yet */
-	 || list_only)
+	/* dry_run is incremented when the destination doesn't exist yet. */
+	if (dry_run > 1 || list_only)
 		return;
 
 	for (j = 0; j < flist->count; j++) {
@@ -358,7 +358,7 @@ void itemize(struct file_struct *file, int ndx, int statret, STRUCT_STAT *st,
 
 	iflags &= 0xffff;
 	if ((iflags & SIGNIFICANT_ITEM_FLAGS || verbose > 1
-	  || log_format_has_i > 1 || (xname && *xname)) && !read_batch) {
+	  || stdout_format_has_i > 1 || (xname && *xname)) && !read_batch) {
 		if (protocol_version >= 29) {
 			if (ndx >= 0)
 				write_int(sock_f_out, ndx);
@@ -367,8 +367,10 @@ void itemize(struct file_struct *file, int ndx, int statret, STRUCT_STAT *st,
 				write_byte(sock_f_out, fnamecmp_type);
 			if (iflags & ITEM_XNAME_FOLLOWS)
 				write_vstring(sock_f_out, xname, strlen(xname));
-		} else if (ndx >= 0)
-			log_item(file, &stats, iflags, xname);
+		} else if (ndx >= 0) {
+			enum logcode code = logfile_format_has_i ? FINFO : FCLIENT;
+			log_item(code, file, &stats, iflags, xname);
+		}
 	}
 }
 
@@ -451,7 +453,7 @@ static void sum_sizes_sqroot(struct sum_struct *sum, int64 len)
 		int64 l;
 		int b = BLOCKSUM_BIAS;
 		for (l = len; l >>= 1; b += 2) {}
-		for (c = blength; c >>= 1 && b; b--) {}
+		for (c = blength; (c >>= 1) && b; b--) {}
 		/* add a bit, subtract rollsum, round up. */
 		s2length = (b + 1 - 32 + 7) / 8; /* --optimize in compiler-- */
 		s2length = MAX(s2length, csum_length);
@@ -628,7 +630,7 @@ static int try_dests_reg(struct file_struct *file, char *fname, int ndx,
 		case 2:
 			if (!unchanged_attrs(file, stp))
 				continue;
-			if ((always_checksum || ignore_times)
+			if (always_checksum && preserve_times
 			 && cmp_time(stp->st_mtime, file->modtime))
 				continue;
 			best_match = j;
@@ -645,29 +647,31 @@ static int try_dests_reg(struct file_struct *file, char *fname, int ndx,
 		j = best_match;
 		pathjoin(cmpbuf, MAXPATHLEN, basis_dir[j], fname);
 		if (link_stat(cmpbuf, stp, 0) < 0)
-			match_level = 0;
+			return -1;
 	}
 
-#ifdef HAVE_LINK
 	if (match_level == 3 && !copy_dest) {
+#ifdef SUPPORT_HARD_LINKS
 		if (link_dest) {
 			if (hard_link_one(file, ndx, fname, 0, stp,
 					  cmpbuf, 1,
 					  itemizing && verbose > 1,
 					  code) < 0)
 				goto try_a_copy;
-			if (preserve_hard_links && file->link_u.links)
+			if (preserve_hard_links && file->link_u.links) {
+				if (dry_run)
+					file->link_u.links->link_dest_used = j + 1;
 				hard_link_cluster(file, ndx, itemizing, code);
-		} else if (itemizing)
+			}
+		} else
+#endif
+		if (itemizing)
 			itemize(file, ndx, 0, stp, 0, 0, NULL);
 		if (verbose > 1 && maybe_ATTRS_REPORT) {
-			code = daemon_log_format_has_i || dry_run
-			     ? FCLIENT : FINFO;
-			rprintf(code, "%s is uptodate\n", fname);
+			rprintf(FCLIENT, "%s is uptodate\n", fname);
 		}
 		return -2;
 	}
-#endif
 
 	if (match_level >= 2) {
 	  try_a_copy: /* Copy the file locally. */
@@ -684,8 +688,7 @@ static int try_dests_reg(struct file_struct *file, char *fname, int ndx,
 		if (maybe_ATTRS_REPORT
 		 && ((!itemizing && verbose && match_level == 2)
 		  || (verbose > 1 && match_level == 3))) {
-			code = daemon_log_format_has_i || dry_run
-			     ? FCLIENT : FINFO;
+			code = match_level == 3 ? FCLIENT : FINFO;
 			rprintf(code, "%s%s\n", fname,
 				match_level == 3 ? " is uptodate" : "");
 		}
@@ -731,9 +734,10 @@ static int try_dests_non(struct file_struct *file, char *fname, int ndx,
 		} else {
 			rprintf(FERROR,
 				"internal: try_dests_non() called with invalid mode (%o)\n",
-				file->mode);
+				(int)file->mode);
 			exit_cleanup(RERR_UNSUPPORTED);
 		}
+#ifdef SUPPORT_HARD_LINKS
 		if (link_dest
 #ifndef CAN_HARDLINK_SYMLINK
 		 && !S_ISLNK(file->mode)
@@ -751,16 +755,15 @@ static int try_dests_non(struct file_struct *file, char *fname, int ndx,
 			if (preserve_hard_links && file->link_u.links)
 				hard_link_cluster(file, ndx, itemizing, code);
 		}
-		if (itemizing && log_format_has_i && verbose > 1) {
+#endif
+		if (itemizing && stdout_format_has_i && verbose > 1) {
 			int changes = compare_dest ? 0 : ITEM_LOCAL_CHANGE
 				    + (link_dest ? ITEM_XNAME_FOLLOWS : 0);
 			char *lp = link_dest ? "" : NULL;
 			itemize(file, ndx, 0, &st, changes, 0, lp);
 		}
 		if (verbose > 1 && maybe_ATTRS_REPORT) {
-			code = daemon_log_format_has_i || dry_run
-			     ? FCLIENT : FINFO;
-			rprintf(code, "%s is uptodate\n", fname);
+			rprintf(FCLIENT, "%s is uptodate\n", fname);
 		}
 		return -2;
 	} while (basis_dir[++i] != NULL);
@@ -807,7 +810,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 			fuzzy_dirlist = NULL;
 		}
 		if (missing_below >= 0) {
-			dry_run--;
+			if (dry_run)
+				dry_run--;
 			missing_below = -1;
 		}
 		parent_dirname = "";
@@ -837,9 +841,13 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		}
 	}
 
-	if (missing_below >= 0 && file->dir.depth <= missing_below) {
-		dry_run--;
-		missing_below = -1;
+	if (missing_below >= 0) {
+		if (file->dir.depth <= missing_below) {
+			if (dry_run)
+				dry_run--;
+			missing_below = -1;
+		} else if (!dry_run)
+			return;
 	}
 	if (dry_run > 1) {
 		statret = -1;
@@ -906,8 +914,14 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 			dry_run++;
 		}
 		if (itemizing && f_out != -1) {
-			itemize(file, ndx, statret, &st,
-				statret ? ITEM_LOCAL_CHANGE : 0, 0, NULL);
+			int sr = statret;
+			if (new_root_dir) {
+				if (*fname == '.' && fname[1] == '\0')
+					sr = -1;
+				new_root_dir = 0;
+			}
+			itemize(file, ndx, sr, &st,
+				sr ? ITEM_LOCAL_CHANGE : 0, 0, NULL);
 		}
 		if (statret != 0 && do_mkdir(fname,file->mode) < 0 && errno != EEXIST) {
 			if (!relative_paths || errno != ENOENT
@@ -916,10 +930,18 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 				rsyserr(FERROR, errno,
 					"recv_generator: mkdir %s failed",
 					full_fname(fname));
+				file->flags |= FLAG_MISSING;
+				if (ndx+1 < the_file_list->count
+				 && the_file_list->files[ndx+1]->dir.depth > file->dir.depth) {
+					rprintf(FERROR,
+					    "*** Skipping everything below this failed directory ***\n");
+					missing_below = file->dir.depth;
+				}
+				return;
 			}
 		}
 		if (set_file_attrs(fname, file, statret ? NULL : &st, 0)
-		    && verbose && code && f_out != -1)
+		    && verbose && code != FNONE && f_out != -1)
 			rprintf(code, "%s/\n", fname);
 		if (delete_during && f_out != -1 && !phase && dry_run < 2
 		    && (file->flags & FLAG_DEL_HERE))
@@ -967,6 +989,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 								  itemizing,
 								  code);
 					}
+					if (remove_source_files == 1)
+						goto return_with_success;
 					return;
 				}
 			}
@@ -986,7 +1010,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 #endif
 				if (!copy_dest)
 					return;
-				itemizing = code = 0;
+				itemizing = 0;
+				code = FNONE;
 			}
 		}
 		if (preserve_hard_links && file->link_u.links
@@ -1002,17 +1027,17 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 				itemize(file, ndx, statret, &st,
 					ITEM_LOCAL_CHANGE, 0, NULL);
 			}
-			if (code && verbose) {
+			if (code != FNONE && verbose) {
 				rprintf(code, "%s -> %s\n", fname,
 					file->u.link);
 			}
-			if (remove_sent_files && !dry_run) {
-				char numbuf[4];
-				SIVAL(numbuf, 0, ndx);
-				send_msg(MSG_SUCCESS, numbuf, 4);
-			}
 			if (preserve_hard_links && file->link_u.links)
 				hard_link_cluster(file, ndx, itemizing, code);
+			/* This does not check remove_source_files == 1
+			 * because this is one of the items that the old
+			 * --remove-sent-files option would remove. */
+			if (remove_source_files)
+				goto return_with_success;
 		}
 #endif
 		return;
@@ -1030,7 +1055,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 #endif
 				if (!copy_dest)
 					return;
-				itemizing = code = 0;
+				itemizing = 0;
+				code = FNONE;
 			}
 		}
 		if (statret != 0
@@ -1060,12 +1086,14 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 					itemize(file, ndx, statret, &st,
 						ITEM_LOCAL_CHANGE, 0, NULL);
 				}
-				if (code && verbose)
+				if (code != FNONE && verbose)
 					rprintf(code, "%s\n", fname);
 				if (preserve_hard_links && file->link_u.links) {
 					hard_link_cluster(file, ndx,
 							  itemizing, code);
 				}
+				if (remove_source_files == 1)
+					goto return_with_success;
 			}
 		} else {
 			if (itemizing)
@@ -1073,6 +1101,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 			set_file_attrs(fname, file, &st, maybe_ATTRS_REPORT);
 			if (preserve_hard_links && file->link_u.links)
 				hard_link_cluster(file, ndx, itemizing, code);
+			if (remove_source_files == 1)
+				goto return_with_success;
 		}
 		return;
 	}
@@ -1127,9 +1157,12 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 	if (statret != 0 && basis_dir[0] != NULL) {
 		int j = try_dests_reg(file, fname, ndx, fnamecmpbuf, &st,
 				      itemizing, maybe_ATTRS_REPORT, code);
-		if (j == -2)
+		if (j == -2) {
+			if (remove_source_files == 1)
+				goto return_with_success;
 			return;
-		if (j != -1) {
+		}
+		if (j >= 0) {
 			fnamecmp = fnamecmpbuf;
 			fnamecmp_type = j;
 			statret = 0;
@@ -1194,6 +1227,14 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		set_file_attrs(fname, file, &st, maybe_ATTRS_REPORT);
 		if (preserve_hard_links && file->link_u.links)
 			hard_link_cluster(file, ndx, itemizing, code);
+		if (remove_source_files != 1)
+			return;
+	  return_with_success:
+		if (!dry_run) {
+			char numbuf[4];
+			SIVAL(numbuf, 0, ndx);
+			send_msg(MSG_SUCCESS, numbuf, 4);
+		}
 		return;
 	}
 
@@ -1266,8 +1307,8 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		rprintf(FINFO, "generating and sending sums for %d\n", ndx);
 
   notify_others:
-	if (remove_sent_files && !delay_updates && !phase)
-	    increment_active_files(ndx, itemizing, code);
+	if (remove_source_files && !delay_updates && !phase)
+		increment_active_files(ndx, itemizing, code);
 	write_int(f_out, ndx);
 	if (itemizing) {
 		int iflags = ITEM_TRANSFER;
@@ -1326,16 +1367,16 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 
 	if (protocol_version >= 29) {
 		itemizing = 1;
-		maybe_ATTRS_REPORT = log_format_has_i ? 0 : ATTRS_REPORT;
-		code = daemon_log_format_has_i ? 0 : FLOG;
+		maybe_ATTRS_REPORT = stdout_format_has_i ? 0 : ATTRS_REPORT;
+		code = logfile_format_has_i ? FNONE : FLOG;
 	} else if (am_daemon) {
-		itemizing = daemon_log_format_has_i && do_xfers;
+		itemizing = logfile_format_has_i && do_xfers;
 		maybe_ATTRS_REPORT = ATTRS_REPORT;
 		code = itemizing || !do_xfers ? FCLIENT : FINFO;
 	} else if (!am_server) {
-		itemizing = log_format_has_i;
-		maybe_ATTRS_REPORT = log_format_has_i ? 0 : ATTRS_REPORT;
-		code = itemizing ? 0 : FINFO;
+		itemizing = stdout_format_has_i;
+		maybe_ATTRS_REPORT = stdout_format_has_i ? 0 : ATTRS_REPORT;
+		code = itemizing ? FNONE : FINFO;
 	} else {
 		itemizing = 0;
 		maybe_ATTRS_REPORT = ATTRS_REPORT;
@@ -1476,6 +1517,16 @@ void generate_files(int f_out, struct file_list *flist, char *local_name)
 				continue;
 			if (!need_retouch_dir_times && file->mode & S_IWUSR)
 				continue;
+			if (file->flags & FLAG_MISSING) {
+				int missing = file->dir.depth;
+				while (++i < flist->count) {
+					file = flist->files[i];
+					if (file->dir.depth <= missing)
+						break;
+				}
+				i--;
+				continue;
+			}
 			recv_generator(f_name(file, NULL), file, i, itemizing,
 				       maybe_ATTRS_REPORT, code, -1);
 			if (allowed_lull && !(++j % lull_mod))
