@@ -1,29 +1,34 @@
 /*
-   Copyright (C) Andrew Tridgell 1996
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2002 by Martin Pool <mbp@samba.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * Routines to support hard-linking.
+ *
+ * Copyright (C) 1996 Andrew Tridgell
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "rsync.h"
 
 extern int verbose;
+extern int do_xfers;
 extern int link_dest;
 extern int make_backups;
-extern int log_format_has_i;
+extern int remove_source_files;
+extern int stdout_format_has_i;
 extern char *basis_dir[];
 extern struct file_list *the_file_list;
 
@@ -50,14 +55,14 @@ static int hlink_compare(int *int1, int *int2)
 	return f_name_cmp(f1, f2);
 }
 
-static int *hlink_list;
-static int hlink_count;
+static int32 *hlink_list;
+static int32 hlink_count;
 
 /* Analyze the data in the hlink_list[], remove items that aren't multiply
  * linked, and replace the dev+inode data with the hlindex+next linked list. */
 static void link_idev_data(void)
 {
-	int cur, from, to, start;
+	int32 cur, from, to, start;
 
 	alloc_pool_t hlink_pool;
 	alloc_pool_t idev_pool = the_file_list->hlink_pool;
@@ -78,6 +83,7 @@ static void link_idev_data(void)
 
 			FPTR(cur)->F_HLINDEX = to;
 			FPTR(cur)->F_NEXT = hlink_list[++from];
+			FPTR(cur)->link_u.links->link_dest_used = 0;
 		}
 		pool_free(idev_pool, 0, FPTR(cur)->link_u.idev);
 		if (from > start) {
@@ -89,6 +95,7 @@ static void link_idev_data(void)
 			FPTR(cur)->F_HLINDEX = to;
 			FPTR(cur)->F_NEXT = head;
 			FPTR(cur)->flags |= FLAG_HLINK_EOL;
+			FPTR(cur)->link_u.links->link_dest_used = 0;
 			hlink_list[to++] = head;
 		} else
 			FPTR(cur)->link_u.links = NULL;
@@ -101,7 +108,7 @@ static void link_idev_data(void)
 		hlink_pool = NULL;
 	} else {
 		hlink_count = to;
-		hlink_list = realloc_array(hlink_list, int, hlink_count);
+		hlink_list = realloc_array(hlink_list, int32, hlink_count);
 		if (!hlink_list)
 			out_of_memory("init_hard_links");
 	}
@@ -118,7 +125,7 @@ void init_hard_links(void)
 	if (hlink_list)
 		free(hlink_list);
 
-	if (!(hlink_list = new_array(int, the_file_list->count)))
+	if (!(hlink_list = new_array(int32, the_file_list->count)))
 		out_of_memory("init_hard_links");
 
 	hlink_count = 0;
@@ -180,13 +187,19 @@ int hard_link_check(struct file_struct *file, int ndx, char *fname,
 		head = hlink_list[file->F_HLINDEX];
 	if (ndx != head) {
 		struct file_struct *head_file = FPTR(head);
-		if (!log_format_has_i && verbose > 1) {
+		if (!stdout_format_has_i && verbose > 1) {
 			rprintf(FINFO, "\"%s\" is a hard link\n",
 				f_name(file, NULL));
 		}
 		if (head_file->F_HLINDEX == FINISHED_LINK) {
 			STRUCT_STAT st2, st3;
-			char *toname = f_name(head_file, NULL);
+			char toname[MAXPATHLEN];
+			int ldu = head_file->link_u.links->link_dest_used;
+			if (ldu) {
+				pathjoin(toname, MAXPATHLEN, basis_dir[ldu-1],
+					 f_name(head_file, NULL));
+			} else
+				f_name(head_file, toname);
 			if (link_stat(toname, &st2, 0) < 0) {
 				rsyserr(FERROR, errno, "stat %s failed",
 					full_fname(toname));
@@ -205,8 +218,10 @@ int hard_link_check(struct file_struct *file, int ndx, char *fname,
 							continue;
 						statret = 1;
 						st = &st3;
-						if (verbose < 2 || !log_format_has_i)
-							itemizing = code = 0;
+						if (verbose < 2 || !stdout_format_has_i) {
+							itemizing = 0;
+							code = FNONE;
+						}
 						break;
 					}
 					if (!unchanged_file(cmpbuf, file, &st3))
@@ -219,6 +234,11 @@ int hard_link_check(struct file_struct *file, int ndx, char *fname,
 			}
 			maybe_hard_link(file, ndx, fname, statret, st,
 					toname, &st2, itemizing, code);
+			if (remove_source_files == 1 && do_xfers) {
+				char numbuf[4];
+				SIVAL(numbuf, 0, ndx);
+				send_msg(MSG_SUCCESS, numbuf, 4);
+			}
 			file->F_HLINDEX = FINISHED_LINK;
 		} else
 			file->F_HLINDEX = SKIPPED_LINK;
@@ -250,7 +270,7 @@ int hard_link_one(struct file_struct *file, int ndx, char *fname,
 			ITEM_LOCAL_CHANGE | ITEM_XNAME_FOLLOWS, 0,
 			terse ? "" : toname);
 	}
-	if (code && verbose && !terse)
+	if (code != FNONE && verbose && !terse)
 		rprintf(code, "%s => %s\n", fname, toname);
 	return 0;
 }
@@ -284,6 +304,11 @@ void hard_link_cluster(struct file_struct *file, int master, int itemizing,
 		statret = link_stat(hlink2, &st2, 0);
 		maybe_hard_link(file, ndx, hlink2, statret, &st2,
 				hlink1, &st1, itemizing, code);
+		if (remove_source_files == 1 && do_xfers) {
+			char numbuf[4];
+			SIVAL(numbuf, 0, ndx);
+			send_msg(MSG_SUCCESS, numbuf, 4);
+		}
 		file->F_HLINDEX = FINISHED_LINK;
 	} while (!(file->flags & FLAG_HLINK_EOL));
 #endif
