@@ -1,7 +1,9 @@
-/*  -*- c-file-style: "linux" -*-
+/*
+ * Command-line (and received via daemon-socket) option parsing.
  *
- * Copyright (C) 1998-2001 by Andrew Tridgell <tridge@samba.org>
- * Copyright (C) 2000, 2001, 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 1998-2001 Andrew Tridgell <tridge@samba.org>
+ * Copyright (C) 2000, 2001, 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,9 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "rsync.h"
@@ -63,7 +65,7 @@ int delete_during = 0;
 int delete_before = 0;
 int delete_after = 0;
 int delete_excluded = 0;
-int remove_sent_files = 0;
+int remove_source_files = 0;
 int one_file_system = 0;
 int protocol_version = PROTOCOL_VERSION;
 int sparse_files = 0;
@@ -145,7 +147,9 @@ char *partial_dir = NULL;
 char *basis_dir[MAX_BASIS_DIRS+1];
 char *config_file = NULL;
 char *shell_cmd = NULL;
-char *log_format = NULL;
+char *logfile_name = NULL;
+char *logfile_format = NULL;
+char *stdout_format = NULL;
 char *password_file = NULL;
 char *rsync_path = RSYNC_PATH;
 char *backup_dir = NULL;
@@ -160,9 +164,12 @@ char *dest_option = NULL;
 
 int verbose = 0;
 int quiet = 0;
+int output_motd = 1;
 int log_before_transfer = 0;
-int log_format_has_i = 0;
-int log_format_has_o_or_i = 0;
+int stdout_format_has_i = 0;
+int stdout_format_has_o_or_i = 0;
+int logfile_format_has_i = 0;
+int logfile_format_has_o_or_i = 0;
 int always_checksum = 0;
 int list_only = 0;
 
@@ -246,7 +253,8 @@ static void print_rsync_version(enum logcode f)
 			(int) SIZEOF_INT64, (int) sizeof (int64));
 	}
 
-	rprintf(f,"\nrsync comes with ABSOLUTELY NO WARRANTY.  This is free software, and you\n");
+	rprintf(f,"\n");
+	rprintf(f,"rsync comes with ABSOLUTELY NO WARRANTY.  This is free software, and you\n");
 	rprintf(f,"are welcome to redistribute it under certain conditions.  See the GNU\n");
 	rprintf(f,"General Public Licence for details.\n");
 }
@@ -256,10 +264,12 @@ void usage(enum logcode F)
 {
   print_rsync_version(F);
 
-  rprintf(F,"\nrsync is a file transfer program capable of efficient remote update\n");
+  rprintf(F,"\n");
+  rprintf(F,"rsync is a file transfer program capable of efficient remote update\n");
   rprintf(F,"via a fast differencing algorithm.\n");
 
-  rprintf(F,"\nUsage: rsync [OPTION]... SRC [SRC]... DEST\n");
+  rprintf(F,"\n");
+  rprintf(F,"Usage: rsync [OPTION]... SRC [SRC]... DEST\n");
   rprintf(F,"  or   rsync [OPTION]... SRC [SRC]... [USER@]HOST:DEST\n");
   rprintf(F,"  or   rsync [OPTION]... SRC [SRC]... [USER@]HOST::DEST\n");
   rprintf(F,"  or   rsync [OPTION]... SRC [SRC]... rsync://[USER@]HOST[:PORT]/DEST\n");
@@ -268,9 +278,11 @@ void usage(enum logcode F)
   rprintf(F,"  or   rsync [OPTION]... rsync://[USER@]HOST[:PORT]/SRC [DEST]\n");
   rprintf(F,"The ':' usages connect via remote shell, while '::' & 'rsync://' usages connect\n");
   rprintf(F,"to an rsync daemon, and require SRC or DEST to start with a module name.\n");
-  rprintf(F,"\nOptions\n");
+  rprintf(F,"\n");
+  rprintf(F,"Options\n");
   rprintf(F," -v, --verbose               increase verbosity\n");
   rprintf(F," -q, --quiet                 suppress non-error messages\n");
+  rprintf(F,"     --no-motd               suppress daemon-mode MOTD (see manpage caveat)\n");
   rprintf(F," -c, --checksum              skip based on checksum, not mod-time & size\n");
   rprintf(F," -a, --archive               archive mode; same as -rlptgoD (no -H)\n");
   rprintf(F,"     --no-OPTION             turn off an implied OPTION (e.g. --no-D)\n");
@@ -293,7 +305,7 @@ void usage(enum logcode F)
   rprintf(F," -H, --hard-links            preserve hard links\n");
   rprintf(F," -p, --perms                 preserve permissions\n");
   rprintf(F," -E, --executability         preserve the file's executability\n");
-  rprintf(F,"     --chmod=CHMOD           change destination permissions\n");
+  rprintf(F,"     --chmod=CHMOD           affect file and/or directory permissions\n");
   rprintf(F," -o, --owner                 preserve owner (super-user only)\n");
   rprintf(F," -g, --group                 preserve group\n");
   rprintf(F,"     --devices               preserve device files (super-user only)\n");
@@ -309,15 +321,15 @@ void usage(enum logcode F)
   rprintf(F," -B, --block-size=SIZE       force a fixed checksum block-size\n");
   rprintf(F," -e, --rsh=COMMAND           specify the remote shell to use\n");
   rprintf(F,"     --rsync-path=PROGRAM    specify the rsync to run on the remote machine\n");
-  rprintf(F,"     --existing              ignore non-existing files on receiving side\n");
-  rprintf(F,"     --ignore-existing       ignore files that already exist on receiving side\n");
-  rprintf(F,"     --remove-sent-files     sent files/symlinks are removed from sending side\n");
+  rprintf(F,"     --existing              skip creating new files on receiver\n");
+  rprintf(F,"     --ignore-existing       skip updating files that already exist on receiver\n");
+  rprintf(F,"     --remove-source-files   sender removes synchronized files (non-dirs)\n");
   rprintf(F,"     --del                   an alias for --delete-during\n");
-  rprintf(F,"     --delete                delete files that don't exist on the sending side\n");
+  rprintf(F,"     --delete                delete extraneous files from destination dirs\n");
   rprintf(F,"     --delete-before         receiver deletes before transfer (default)\n");
   rprintf(F,"     --delete-during         receiver deletes during transfer, not before\n");
   rprintf(F,"     --delete-after          receiver deletes after transfer, not before\n");
-  rprintf(F,"     --delete-excluded       also delete excluded files on the receiving side\n");
+  rprintf(F,"     --delete-excluded       also delete excluded files from destination dirs\n");
   rprintf(F,"     --ignore-errors         delete even if there are I/O errors\n");
   rprintf(F,"     --force                 force deletion of directories even if not empty\n");
   rprintf(F,"     --max-delete=NUM        don't delete more than NUM files\n");
@@ -359,7 +371,9 @@ void usage(enum logcode F)
   rprintf(F,"     --progress              show progress during transfer\n");
   rprintf(F," -P                          same as --partial --progress\n");
   rprintf(F," -i, --itemize-changes       output a change-summary for all updates\n");
-  rprintf(F,"     --log-format=FORMAT     output filenames using the specified format\n");
+  rprintf(F,"     --out-format=FORMAT     output updates using the specified FORMAT\n");
+  rprintf(F,"     --log-file=FILE         log what we're doing to the specified FILE\n");
+  rprintf(F,"     --log-file-format=FMT   log updates using the specified FMT\n");
   rprintf(F,"     --password-file=FILE    read password from FILE\n");
   rprintf(F,"     --list-only             list the files instead of copying them\n");
   rprintf(F,"     --bwlimit=KBPS          limit I/O bandwidth; KBytes per second\n");
@@ -374,7 +388,8 @@ void usage(enum logcode F)
   rprintf(F,"     --version               print version number\n");
   rprintf(F,"(-h) --help                  show this help (-h works with no other options)\n");
 
-  rprintf(F,"\nUse \"rsync --daemon --help\" to see the daemon-mode command-line options.\n");
+  rprintf(F,"\n");
+  rprintf(F,"Use \"rsync --daemon --help\" to see the daemon-mode command-line options.\n");
   rprintf(F,"Please see the rsync(1) and rsyncd.conf(5) man pages for full documentation.\n");
   rprintf(F,"See http://rsync.samba.org/ for updates, bug reports, and answers\n");
 }
@@ -394,6 +409,8 @@ static struct poptOption long_options[] = {
   {"no-verbose",       0,  POPT_ARG_VAL,    &verbose, 0, 0, 0 },
   {"no-v",             0,  POPT_ARG_VAL,    &verbose, 0, 0, 0 },
   {"quiet",           'q', POPT_ARG_NONE,   0, 'q', 0, 0 },
+  {"motd",             0,  POPT_ARG_VAL,    &output_motd, 1, 0, 0 },
+  {"no-motd",          0,  POPT_ARG_VAL,    &output_motd, 0, 0, 0 },
   {"stats",            0,  POPT_ARG_NONE,   &do_stats, 0, 0, 0 },
   {"human-readable",  'h', POPT_ARG_NONE,   0, 'h', 0, 0},
   {"dry-run",         'n', POPT_ARG_NONE,   &dry_run, 0, 0, 0 },
@@ -462,7 +479,8 @@ static struct poptOption long_options[] = {
   {"delete-during",    0,  POPT_ARG_NONE,   &delete_during, 0, 0, 0 },
   {"delete-after",     0,  POPT_ARG_NONE,   &delete_after, 0, 0, 0 },
   {"delete-excluded",  0,  POPT_ARG_NONE,   &delete_excluded, 0, 0, 0 },
-  {"remove-sent-files",0,  POPT_ARG_NONE,   &remove_sent_files, 0, 0, 0 },
+  {"remove-sent-files",0,  POPT_ARG_VAL,    &remove_source_files, 2, 0, 0 }, /* deprecated */
+  {"remove-source-files",0,POPT_ARG_VAL,    &remove_source_files, 1, 0, 0 },
   {"force",            0,  POPT_ARG_NONE,   &force_delete, 0, 0, 0 },
   {"ignore-errors",    0,  POPT_ARG_NONE,   &ignore_errors, 0, 0, 0 },
   {"max-delete",       0,  POPT_ARG_INT,    &max_delete, 0, 0, 0 },
@@ -492,7 +510,10 @@ static struct poptOption long_options[] = {
   {"partial-dir",      0,  POPT_ARG_STRING, &partial_dir, 0, 0, 0 },
   {"delay-updates",    0,  POPT_ARG_NONE,   &delay_updates, 0, 0, 0 },
   {"prune-empty-dirs",'m', POPT_ARG_NONE,   &prune_empty_dirs, 0, 0, 0 },
-  {"log-format",       0,  POPT_ARG_STRING, &log_format, 0, 0, 0 },
+  {"log-file",         0,  POPT_ARG_STRING, &logfile_name, 0, 0, 0 },
+  {"log-file-format",  0,  POPT_ARG_STRING, &logfile_format, 0, 0, 0 },
+  {"out-format",       0,  POPT_ARG_STRING, &stdout_format, 0, 0, 0 },
+  {"log-format",       0,  POPT_ARG_STRING, &stdout_format, 0, 0, 0 }, /* DEPRECATED */
   {"itemize-changes", 'i', POPT_ARG_NONE,   0, 'i', 0, 0 },
   {"bwlimit",          0,  POPT_ARG_INT,    &bwlimit, 0, 0, 0 },
   {"backup",          'b', POPT_ARG_NONE,   &make_backups, 0, 0, 0 },
@@ -536,12 +557,15 @@ static void daemon_usage(enum logcode F)
 {
   print_rsync_version(F);
 
-  rprintf(F,"\nUsage: rsync --daemon [OPTION]...\n");
+  rprintf(F,"\n");
+  rprintf(F,"Usage: rsync --daemon [OPTION]...\n");
   rprintf(F,"     --address=ADDRESS       bind to the specified address\n");
   rprintf(F,"     --bwlimit=KBPS          limit I/O bandwidth; KBytes per second\n");
   rprintf(F,"     --config=FILE           specify alternate rsyncd.conf file\n");
   rprintf(F,"     --no-detach             do not detach from the parent\n");
   rprintf(F,"     --port=PORT             listen on alternate port number\n");
+  rprintf(F,"     --log-file=FILE         override the \"log file\" setting\n");
+  rprintf(F,"     --log-file-format=FMT   override the \"log format\" setting\n");
   rprintf(F,"     --sockopts=OPTIONS      specify custom TCP options\n");
   rprintf(F," -v, --verbose               increase verbosity\n");
 #ifdef INET6
@@ -550,7 +574,8 @@ static void daemon_usage(enum logcode F)
 #endif
   rprintf(F,"     --help                  show this help screen\n");
 
-  rprintf(F,"\nIf you were not trying to invoke rsync as a daemon, avoid using any of the\n");
+  rprintf(F,"\n");
+  rprintf(F,"If you were not trying to invoke rsync as a daemon, avoid using any of the\n");
   rprintf(F,"daemon-specific rsync options.  See also the rsyncd.conf(5) man page.\n");
 }
 
@@ -565,6 +590,8 @@ static struct poptOption long_daemon_options[] = {
   {"ipv6",            '6', POPT_ARG_VAL,    &default_af_hint, AF_INET6, 0, 0 },
 #endif
   {"detach",           0,  POPT_ARG_VAL,    &no_detach, 0, 0, 0 },
+  {"log-file",         0,  POPT_ARG_STRING, &logfile_name, 0, 0, 0 },
+  {"log-file-format",  0,  POPT_ARG_STRING, &logfile_format, 0, 0, 0 },
   {"no-detach",        0,  POPT_ARG_VAL,    &no_detach, 1, 0, 0 },
   {"port",             0,  POPT_ARG_INT,    &rsync_port, 0, 0, 0 },
   {"sockopts",         0,  POPT_ARG_STRING, &sockopts, 0, 0, 0 },
@@ -590,8 +617,9 @@ static char err_buf[200];
 void option_error(void)
 {
 	if (!err_buf[0]) {
-		strcpy(err_buf, "Error parsing options: "
-		    "option may be supported on client but not on server?\n");
+		strlcpy(err_buf, "Error parsing options: option may "
+			"be supported on client but not on server?\n",
+			sizeof err_buf);
 	}
 
 	rprintf(FERROR, RSYNC_NAME ": %s", err_buf);
@@ -665,17 +693,6 @@ static void set_refuse_options(char *bp)
 		*cp = ' ';
 		bp = cp + 1;
 	}
-
-	for (op = long_options; ; op++) {
-		*shortname = op->shortName;
-		if (!op->longName && !*shortname)
-			break;
-		if (op->val == OPT_DAEMON) {
-			if (op->argInfo == POPT_ARG_VAL)
-				op->argInfo = POPT_ARG_NONE;
-			op->val = (op - long_options) + OPT_REFUSED_BASE;
-		}
-	}
 }
 
 
@@ -780,6 +797,8 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 
 	if (ref && *ref)
 		set_refuse_options(ref);
+	if (am_daemon)
+		set_refuse_options("log-file*");
 
 	/* TODO: Call poptReadDefaultConfig; handle errors. */
 
@@ -818,7 +837,9 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 
 		case OPT_DAEMON:
 			if (am_daemon) {
-				strcpy(err_buf, "Attempt to hack rsync thwarted!\n");
+				strlcpy(err_buf,
+					"Attempt to hack rsync thwarted!\n",
+					sizeof err_buf);
 				return 0;
 			}
 			poptFreeContext(pc);
@@ -889,14 +910,17 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 		case OPT_INCLUDE_FROM:
 			arg = poptGetOptArg(pc);
 			if (sanitize_paths)
-				arg = sanitize_path(NULL, arg, NULL, 0);
+				arg = sanitize_path(NULL, arg, NULL, 0, NULL);
 			if (server_filter_list.head) {
-				char *cp = (char *)arg;
+				char *cp = strdup(arg);
+				if (!cp)
+					out_of_memory("parse_arguments");
 				if (!*cp)
 					goto options_rejected;
 				clean_fname(cp, 1);
 				if (check_filter(&server_filter_list, cp, 0) < 0)
 					goto options_rejected;
+				free(cp);
 			}
 			parse_filter_file(&filter_list, arg,
 				opt == OPT_INCLUDE_FROM ? MATCHFLG_INCLUDE : 0,
@@ -1020,7 +1044,7 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 			break;
 
 		case OPT_LINK_DEST:
-#ifdef HAVE_LINK
+#ifdef SUPPORT_HARD_LINKS
 			link_dest = 1;
 			dest_option = "--link-dest";
 			goto set_dest_dir;
@@ -1046,10 +1070,9 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 					MAX_BASIS_DIRS, dest_option);
 				return 0;
 			}
-			arg = poptGetOptArg(pc);
-			if (sanitize_paths)
-				arg = sanitize_path(NULL, arg, NULL, 0);
-			basis_dir[basis_dir_cnt++] = (char *)arg;
+			/* We defer sanitizing this arg until we know what
+			 * our destination directory is going to be. */
+			basis_dir[basis_dir_cnt++] = (char *)poptGetOptArg(pc);
 			break;
 
 		case OPT_CHMOD:
@@ -1187,8 +1210,8 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 		return 0;
 	}
 
-	if (remove_sent_files) {
-		/* We only want to infer this refusal of --remove-sent-files
+	if (remove_source_files) {
+		/* We only want to infer this refusal of --remove-source-files
 		 * via the refusal of "delete", not any of the "delete-FOO"
 		 * options. */
 		if (refused_delete && am_sender) {
@@ -1204,17 +1227,14 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 	if (sanitize_paths) {
 		int i;
 		for (i = *argc; i-- > 0; )
-			(*argv)[i] = sanitize_path(NULL, (*argv)[i], "", 0);
+			(*argv)[i] = sanitize_path(NULL, (*argv)[i], "", 0, NULL);
 		if (tmpdir)
-			tmpdir = sanitize_path(NULL, tmpdir, NULL, 0);
-		if (partial_dir)
-			partial_dir = sanitize_path(NULL, partial_dir, NULL, 0);
+			tmpdir = sanitize_path(NULL, tmpdir, NULL, 0, NULL);
 		if (backup_dir)
-			backup_dir = sanitize_path(NULL, backup_dir, NULL, 0);
+			backup_dir = sanitize_path(NULL, backup_dir, NULL, 0, NULL);
 	}
 	if (server_filter_list.head && !am_sender) {
 		struct filter_list_struct *elp = &server_filter_list;
-		int i;
 		if (tmpdir) {
 			if (!*tmpdir)
 				goto options_rejected;
@@ -1222,18 +1242,6 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 			if (check_filter(elp, tmpdir, 1) < 0)
 				goto options_rejected;
 		}
-		if (partial_dir && *partial_dir) {
-			clean_fname(partial_dir, 1);
-			if (check_filter(elp, partial_dir, 1) < 0)
-				goto options_rejected;
-		}
-		for (i = 0; i < basis_dir_cnt; i++) {
-			if (!*basis_dir[i])
-				goto options_rejected;
-			clean_fname(basis_dir[i], 1);
-			if (check_filter(elp, basis_dir[i], 1) < 0)
-				goto options_rejected;
-		}
 		if (backup_dir) {
 			if (!*backup_dir)
 				goto options_rejected;
@@ -1282,17 +1290,17 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 	if (make_backups && !backup_dir)
 		omit_dir_times = 1;
 
-	if (log_format) {
-		if (am_server && log_format_has(log_format, 'I'))
-			log_format_has_i = 2;
-		else if (log_format_has(log_format, 'i'))
-			log_format_has_i = itemize_changes | 1;
-		if (!log_format_has(log_format, 'b')
-		 && !log_format_has(log_format, 'c'))
+	if (stdout_format) {
+		if (am_server && log_format_has(stdout_format, 'I'))
+			stdout_format_has_i = 2;
+		else if (log_format_has(stdout_format, 'i'))
+			stdout_format_has_i = itemize_changes | 1;
+		if (!log_format_has(stdout_format, 'b')
+		 && !log_format_has(stdout_format, 'c'))
 			log_before_transfer = !am_server;
 	} else if (itemize_changes) {
-		log_format = "%i %n%L";
-		log_format_has_i = itemize_changes;
+		stdout_format = "%i %n%L";
+		stdout_format_has_i = itemize_changes;
 		log_before_transfer = !am_server;
 	}
 
@@ -1304,12 +1312,26 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 
 	set_io_timeout(io_timeout);
 
-	if (verbose && !log_format) {
-		log_format = "%n%L";
+	if (verbose && !stdout_format) {
+		stdout_format = "%n%L";
 		log_before_transfer = !am_server;
 	}
-	if (log_format_has_i || log_format_has(log_format, 'o'))
-		log_format_has_o_or_i = 1;
+	if (stdout_format_has_i || log_format_has(stdout_format, 'o'))
+		stdout_format_has_o_or_i = 1;
+
+	if (logfile_name && !am_daemon) {
+		if (!logfile_format) {
+			logfile_format = "%i %n%L";
+			logfile_format_has_i = logfile_format_has_o_or_i = 1;
+		} else {
+			if (log_format_has(logfile_format, 'i'))
+				logfile_format_has_i = 1;
+			if (logfile_format_has_i || log_format_has(logfile_format, 'o'))
+				logfile_format_has_o_or_i = 1;
+		}
+		log_init(0);
+	} else if (!am_daemon)
+		logfile_format = NULL;
 
 	if (daemon_bwlimit && (!bwlimit || bwlimit > daemon_bwlimit))
 		bwlimit = daemon_bwlimit;
@@ -1376,7 +1398,7 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 				clean_fname(partial_dir, 1);
 			if (!*partial_dir || strcmp(partial_dir, ".") == 0)
 				partial_dir = NULL;
-			else if (*partial_dir != '/') {
+			else if (*partial_dir != '/' && !am_server) {
 				parse_rule(&filter_list, partial_dir,
 				    MATCHFLG_NO_PREFIXES|MATCHFLG_DIRECTORY, 0);
 			}
@@ -1414,7 +1436,7 @@ int parse_arguments(int *argc, const char ***argv, int frommain)
 			}
 		} else {
 			if (sanitize_paths)
-				files_from = sanitize_path(NULL, files_from, NULL, 0);
+				files_from = sanitize_path(NULL, files_from, NULL, 0, NULL);
 			if (server_filter_list.head) {
 				if (!*files_from)
 					goto options_rejected;
@@ -1484,7 +1506,7 @@ void server_options(char **args,int *argc)
 		argstr[x++] = 'n';
 	if (preserve_links)
 		argstr[x++] = 'l';
-	if (xfer_dirs > (recurse || !delete_mode || !am_sender))
+	if (xfer_dirs > (recurse || !delete_mode || !am_sender ? 1 : 0))
 		argstr[x++] = 'd';
 	if (am_sender) {
 		if (keep_dirlinks)
@@ -1574,12 +1596,13 @@ void server_options(char **args,int *argc)
 
 	/* The server side doesn't use our log-format, but in certain
 	 * circumstances they need to know a little about the option. */
-	if (log_format && am_sender) {
-		if (log_format_has_i > 1)
+	if (stdout_format && am_sender) {
+		/* Use --log-format, not --out-format, for compatibility. */
+		if (stdout_format_has_i > 1)
 			args[ac++] = "--log-format=%i%I";
-		else if (log_format_has_i)
+		else if (stdout_format_has_i)
 			args[ac++] = "--log-format=%i";
-		else if (log_format_has_o_or_i)
+		else if (stdout_format_has_o_or_i)
 			args[ac++] = "--log-format=%o";
 		else if (!verbose)
 			args[ac++] = "--log-format=X";
@@ -1735,7 +1758,9 @@ void server_options(char **args,int *argc)
 	if (fuzzy_basis && am_sender)
 		args[ac++] = "--fuzzy";
 
-	if (remove_sent_files)
+	if (remove_source_files == 1)
+		args[ac++] = "--remove-source-files";
+	else if (remove_source_files)
 		args[ac++] = "--remove-sent-files";
 
 	*argc = ac;
@@ -1756,10 +1781,10 @@ char *check_for_hostspec(char *s, char **host_ptr, int *port_ptr)
 {
 	char *p;
 	int not_host;
+	int hostlen;
 
 	if (port_ptr && strncasecmp(URL_PREFIX, s, strlen(URL_PREFIX)) == 0) {
 		char *path;
-		int hostlen;
 		s += strlen(URL_PREFIX);
 		if ((p = strchr(s, '/')) != NULL) {
 			hostlen = p - s;
@@ -1788,6 +1813,7 @@ char *check_for_hostspec(char *s, char **host_ptr, int *port_ptr)
 
 	if (*s == '[' && (p = strchr(s, ']')) != NULL && p[1] == ':') {
 		s++;
+		hostlen = p - s;
 		*p = '\0';
 		not_host = strchr(s, '/') || !strchr(s, ':');
 		*p = ']';
@@ -1797,6 +1823,7 @@ char *check_for_hostspec(char *s, char **host_ptr, int *port_ptr)
 	} else {
 		if (!(p = strchr(s, ':')))
 			return NULL;
+		hostlen = p - s;
 		*p = '\0';
 		not_host = strchr(s, '/') != NULL;
 		*p = ':';
@@ -1804,8 +1831,8 @@ char *check_for_hostspec(char *s, char **host_ptr, int *port_ptr)
 			return NULL;
 	}
 
-	*host_ptr = new_array(char, p - s + 1);
-	strlcpy(*host_ptr, s, p - s + 1);
+	*host_ptr = new_array(char, hostlen + 1);
+	strlcpy(*host_ptr, s, hostlen + 1);
 
 	if (p[1] == ':') {
 		if (port_ptr && !*port_ptr)
