@@ -1,33 +1,34 @@
-/* -*- c-file-style: "linux" -*-
-
-   Copyright (C) 1996-2000 by Andrew Tridgell
-   Copyright (C) Paul Mackerras 1996
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+/*
+ * Routines only used by the receiving process.
+ *
+ * Copyright (C) 1996-2000 Andrew Tridgell
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "rsync.h"
 
 extern int verbose;
 extern int do_xfers;
-extern int am_daemon;
 extern int am_server;
 extern int do_progress;
 extern int log_before_transfer;
-extern int log_format_has_i;
-extern int daemon_log_format_has_i;
+extern int stdout_format_has_i;
+extern int logfile_format_has_i;
 extern int csum_length;
 extern int read_batch;
 extern int write_batch;
@@ -39,7 +40,7 @@ extern int preserve_perms;
 extern int basis_dir_cnt;
 extern int make_backups;
 extern int cleanup_got_literal;
-extern int remove_sent_files;
+extern int remove_source_files;
 extern int append_mode;
 extern int sparse_files;
 extern int keep_partial;
@@ -47,7 +48,7 @@ extern int checksum_seed;
 extern int inplace;
 extern int delay_updates;
 extern struct stats stats;
-extern char *log_format;
+extern char *stdout_format;
 extern char *tmpdir;
 extern char *partial_dir;
 extern char *basis_dir[];
@@ -56,6 +57,8 @@ extern struct filter_list_struct server_filter_list;
 
 static struct bitbag *delayed_bits = NULL;
 static int phase = 0;
+/* We're either updating the basis file or an identical copy: */
+static int updating_basis;
 
 
 /*
@@ -81,15 +84,13 @@ static int phase = 0;
 
 static int get_tmpname(char *fnametmp, char *fname)
 {
+	int maxname, added, length = 0;
 	char *f;
-	int     length = 0;
-	int	maxname;
 
 	if (tmpdir) {
 		/* Note: this can't overflow, so the return value is safe */
 		length = strlcpy(fnametmp, tmpdir, MAXPATHLEN - 2);
 		fnametmp[length++] = '/';
-		fnametmp[length] = '\0';	/* always NULL terminated */
 	}
 
 	if ((f = strrchr(fname, '/')) != NULL) {
@@ -102,8 +103,9 @@ static int get_tmpname(char *fnametmp, char *fname)
 	} else
 		f = fname;
 	fnametmp[length++] = '.';
-	fnametmp[length] = '\0';		/* always NULL terminated */
 
+	/* The maxname value is bufsize, and includes space for the '\0'.
+	 * (Note that NAME_MAX get -8 for the leading '.' above.) */
 	maxname = MIN(MAXPATHLEN - 7 - length, NAME_MAX - 8);
 
 	if (maxname < 1) {
@@ -112,8 +114,10 @@ static int get_tmpname(char *fnametmp, char *fname)
 		return 0;
 	}
 
-	strlcpy(fnametmp + length, f, maxname);
-	strcat(fnametmp + length, ".XXXXXX");
+	added = strlcpy(fnametmp + length, f, maxname);
+	if (added >= maxname)
+		added = maxname - 1;
+	memcpy(fnametmp + length + added, ".XXXXXX", 8);
 
 	return 1;
 }
@@ -166,9 +170,9 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 			sum_update(map_ptr(mapbuf, offset, len), len);
 			offset = sum.flength;
 		}
-		if (fd != -1 && do_lseek(fd, offset, SEEK_SET) != offset) {
-			rsyserr(FERROR, errno, "lseek failed on %s",
-				full_fname(fname));
+		if (fd != -1 && (j = do_lseek(fd, offset, SEEK_SET)) != offset) {
+			rsyserr(FERROR, errno, "lseek of %s returned %.0f, not %.0f",
+				full_fname(fname), (double)j, (double)offset);
 			exit_cleanup(RERR_FILEIO);
 		}
 	}
@@ -215,15 +219,17 @@ static int receive_data(int f_in, char *fname_r, int fd_r, OFF_T size_r,
 			sum_update(map, len);
 		}
 
-		if (inplace) {
+		if (updating_basis) {
 			if (offset == offset2 && fd != -1) {
+				OFF_T pos;
 				if (flush_write_file(fd) < 0)
 					goto report_write_error;
 				offset += len;
-				if (do_lseek(fd, len, SEEK_CUR) != offset) {
+				if ((pos = do_lseek(fd, len, SEEK_CUR)) != offset) {
 					rsyserr(FERROR, errno,
-						"lseek failed on %s",
-						full_fname(fname));
+						"lseek of %s returned %.0f, not %.0f",
+						full_fname(fname),
+						(double)pos, (double)offset);
 					exit_cleanup(RERR_FILEIO);
 				}
 				continue;
@@ -293,7 +299,7 @@ static void handle_delayed_updates(struct file_list *flist, char *local_name)
 					"rename failed for %s (from %s)",
 					full_fname(fname), partialptr);
 			} else {
-				if (remove_sent_files
+				if (remove_source_files
 				    || (preserve_hard_links
 				     && file->link_u.links)) {
 					SIVAL(numbuf, 0, i);
@@ -341,8 +347,8 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 	struct file_struct *file;
 	struct stats initial_stats;
 	int save_make_backups = make_backups;
-	int itemizing = am_daemon ? daemon_log_format_has_i
-		      : !am_server && log_format_has_i;
+	int itemizing = am_server ? logfile_format_has_i : stdout_format_has_i;
+	enum logcode log_code = log_before_transfer ? FLOG : FINFO;
 	int max_phase = protocol_version >= 29 ? 2 : 1;
 	int i, recv_ok;
 
@@ -357,6 +363,8 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 	if (delay_updates)
 		delayed_bits = bitbag_create(flist->count);
 
+	updating_basis = inplace;
+
 	while (1) {
 		cleanup_disable();
 
@@ -418,14 +426,13 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 		}
 
 		if (!do_xfers) { /* log the transfer */
-			if (!am_server && log_format)
-				log_item(file, &stats, iflags, NULL);
+			log_item(FCLIENT, file, &stats, iflags, NULL);
 			if (read_batch)
 				discard_receive_data(f_in, file->length);
 			continue;
 		}
 		if (write_batch < 0) {
-			log_item(file, &stats, iflags, NULL);
+			log_item(FINFO, file, &stats, iflags, NULL);
 			if (!am_server)
 				discard_receive_data(f_in, file->length);
 			continue;
@@ -457,6 +464,7 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 				fnamecmp = get_backup_name(fname);
 				break;
 			case FNAMECMP_FUZZY:
+				updating_basis = 0;
 				if (file->dirname) {
 					pathjoin(fnamecmpbuf, MAXPATHLEN,
 						 file->dirname, xname);
@@ -465,6 +473,7 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 					fnamecmp = xname;
 				break;
 			default:
+				updating_basis = 0;
 				if (fnamecmp_type >= basis_dir_cnt) {
 					rprintf(FERROR,
 						"invalid basis_dir index: %d.\n",
@@ -511,7 +520,10 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 			}
 		}
 
-		if (fd1 != -1 && do_fstat(fd1,&st) != 0) {
+		if (fd1 == -1) {
+			st.st_mode = 0;
+			st.st_size = 0;
+		} else if (do_fstat(fd1,&st) != 0) {
 			rsyserr(FERROR, errno, "fstat %s failed",
 				full_fname(fnamecmp));
 			discard_receive_data(f_in, file->length);
@@ -544,7 +556,7 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 			file->mode = dest_mode(file->mode, st.st_mode, exists);
 		}
 
-		/* We now check to see if we are writing file "inplace" */
+		/* We now check to see if we are writing the file "inplace" */
 		if (inplace)  {
 			fd2 = do_open(fname, O_WRONLY|O_CREAT, 0600);
 			if (fd2 == -1) {
@@ -589,13 +601,12 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 				continue;
 			}
 
-			if (keep_partial)
-				cleanup_set(fnametmp, partialptr, file, fd1, fd2);
+			cleanup_set(fnametmp, partialptr, file, fd1, fd2);
 		}
 
 		/* log the transfer */
 		if (log_before_transfer)
-			log_item(file, &initial_stats, iflags, NULL);
+			log_item(FCLIENT, file, &initial_stats, iflags, NULL);
 		else if (!am_server && verbose && do_progress)
 			rprintf(FINFO, "%s\n", fname);
 
@@ -603,8 +614,7 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 		recv_ok = receive_data(f_in, fnamecmp, fd1, st.st_size,
 				       fname, fd2, file->length);
 
-		if (!log_before_transfer)
-			log_item(file, &initial_stats, iflags, NULL);
+		log_item(log_code, file, &initial_stats, iflags, NULL);
 
 		if (fd1 != -1)
 			close(fd1);
@@ -615,9 +625,14 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 		}
 
 		if ((recv_ok && (!delay_updates || !partialptr)) || inplace) {
-			if (partialptr == fname || *partial_dir == '/')
-				partialptr = NULL;
-			finish_transfer(fname, fnametmp, partialptr,
+			char *temp_copy_name;
+			if (partialptr == fname)
+				partialptr = temp_copy_name = NULL;
+			else if (*partial_dir == '/')
+				temp_copy_name = NULL;
+			else
+				temp_copy_name = partialptr;
+			finish_transfer(fname, fnametmp, temp_copy_name,
 					file, recv_ok, 1);
 			if (fnamecmp == partialptr) {
 				do_unlink(partialptr);
@@ -639,7 +654,7 @@ int recv_files(int f_in, struct file_list *flist, char *local_name)
 		cleanup_disable();
 
 		if (recv_ok > 0) {
-			if (remove_sent_files
+			if (remove_source_files
 			    || (preserve_hard_links && file->link_u.links)) {
 				SIVAL(numbuf, 0, i);
 				send_msg(MSG_SUCCESS, numbuf, 4);
