@@ -1,23 +1,23 @@
 /*
-   Copyright (C) by Andrew Tridgell 1996, 2000
-   Copyright (C) Paul Mackerras 1996
-   Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
-
+ * Copyright (C) 1996, 2000 Andrew Tridgell
+ * Copyright (C) 1996 Paul Mackerras
+ * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
+ * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #define False 0
 #define True 1
@@ -64,6 +64,7 @@
 #define FLAG_DEL_HERE (1<<3)	/* receiver/generator */
 #define FLAG_HLINK_TOL (1<<4)	/* receiver/generator */
 #define FLAG_NO_FUZZY (1<<5)	/* generator */
+#define FLAG_MISSING (1<<6)	/* generator */
 
 /* update this if you make incompatible changes */
 #define PROTOCOL_VERSION 29
@@ -157,10 +158,10 @@
 	ITEM_BASIS_TYPE_FOLLOWS | ITEM_XNAME_FOLLOWS | ITEM_LOCAL_CHANGE))
 
 
-/* Log-message categories.  Only FERROR and FINFO get sent over the socket.
- * FLOG and FCLIENT are only used on the daemon side for custom logging,
- * while FNAME is only used on the client side. */
-enum logcode { FERROR=1, FINFO=2, FLOG=3, FCLIENT=4, FNAME=5, FSOCKERR=6 };
+/* Log-message categories.  Only FERROR and FINFO get sent over the socket,
+ * but FLOG and FSOCKERR can be sent over the receiver -> generator pipe.
+ * FLOG only goes to the log file, not the client; FCLIENT is the opposite. */
+enum logcode { FNONE=0, FERROR=1, FINFO=2, FLOG=3, FCLIENT=4, FSOCKERR=5 };
 
 /* Messages types that are sent over the message channel.  The logcode
  * values must all be present here with identical numbers. */
@@ -313,6 +314,12 @@ enum msgcode {
 #include <sys/sysmacros.h>
 #endif
 
+#ifdef MAKEDEV_TAKES_3_ARGS
+#define MAKEDEV(devmajor,devminor) makedev(0,devmajor,devminor)
+#else
+#define MAKEDEV(devmajor,devminor) makedev(devmajor,devminor)
+#endif
+
 #ifdef HAVE_COMPAT_H
 #include <compat.h>
 #endif
@@ -382,10 +389,7 @@ enum msgcode {
  * to ensure that any code that really requires a 64-bit integer has
  * it (e.g. the checksum code uses two 32-bit integers for its 64-bit
  * counter). */
-#if SIZEOF_OFF64_T == 8
-# define int64 off64_t
-# define SIZEOF_INT64 8
-#elif SIZEOF_LONG == 8
+#if SIZEOF_LONG == 8
 # define int64 long
 # define SIZEOF_INT64 8
 #elif SIZEOF_INT == 8
@@ -394,6 +398,9 @@ enum msgcode {
 #elif SIZEOF_LONG_LONG == 8
 # define int64 long long
 # define SIZEOF_INT64 8
+#elif SIZEOF_OFF64_T == 8
+# define int64 off64_t
+# define SIZEOF_INT64 8
 #elif SIZEOF_OFF_T == 8
 # define int64 off_t
 # define SIZEOF_INT64 8
@@ -491,8 +498,9 @@ struct idev {
 #define HL_SKIP		1
 
 struct hlink {
-	int next;
-	int hlindex;
+	int32 next;
+	int32 hlindex;
+	unsigned short link_dest_used;
 };
 
 #define F_DEV	link_u.idev->dev
@@ -659,6 +667,7 @@ struct chmod_mode_struct;
 #endif
 
 #define UNUSED(x) x __attribute__((__unused__))
+#define NORETURN __attribute__((__noreturn__))
 
 #include "proto.h"
 
@@ -848,6 +857,9 @@ size_t strlcat(char *d, const char *s, size_t bufsize);
 #ifndef WEXITSTATUS
 #define	WEXITSTATUS(stat)	((int)(((stat)>>8)&0xFF))
 #endif
+#ifndef WIFEXITED
+#define	WIFEXITED(stat)		((int)((stat)&0xFF) == 0)
+#endif
 
 #define exit_cleanup(code) _exit_cleanup(code, __FILE__, __LINE__)
 
