@@ -1,6 +1,7 @@
 #! /bin/sh
 
 # Copyright (C) 2001, 2002 by Martin Pool <mbp@samba.org>
+# Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version
@@ -13,8 +14,7 @@
 # 
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 # rsync top-level test script -- this invokes all the other more
 # detailed tests in order.  This script can either be called by `make
@@ -138,9 +138,27 @@ if [ "x$loglevel" != x ] && [ "$loglevel" -gt 8 ]; then
     fi
 fi
 
+POSIXLY_CORRECT=1 
+if test x"$TOOLDIR" = x; then
+    TOOLDIR=`pwd`
+fi
+srcdir=`dirname $0`
+if test x"$srcdir" = x -o x"$srcdir" = x.; then
+    srcdir="$TOOLDIR"
+fi
+if test x"$rsync_bin" = x; then
+    rsync_bin="$TOOLDIR/rsync"
+fi
+
+# This allows the user to specify extra rsync options -- use carefully!
+RSYNC="$rsync_bin $*"
+#RSYNC="valgrind $rsync_bin $*"
+
+export POSIXLY_CORRECT TOOLDIR srcdir RSYNC
+
 echo "============================================================"
-echo "$0 running in `pwd`"
-echo "    rsync_bin=$rsync_bin"
+echo "$0 running in $TOOLDIR"
+echo "    rsync_bin=$RSYNC"
 echo "    srcdir=$srcdir"
 
 if [ -f /usr/bin/whoami ]; then
@@ -164,7 +182,7 @@ else
 fi    
 
 # Check if setfacl is around and if it supports the -k or -s option.
-if setfacl --help 2>/dev/null | grep ' -k,' >/dev/null; then
+if setfacl --help 2>&1 | grep ' -k,\|\[-[a-z]*k' >/dev/null; then
     setfacl_nodef='setfacl -k'
 elif setfacl -s u::7,g::5,o:5 testsuite 2>/dev/null; then
     setfacl_nodef='setfacl -s u::7,g::5,o:5'
@@ -172,6 +190,8 @@ else
     setfacl_nodef=true
 fi
 
+export setfacl_nodef
+
 if [ ! -f "$rsync_bin" ]; then
     echo "rsync_bin $rsync_bin is not a file" >&2
     exit 2
@@ -182,11 +202,6 @@ if [ ! -d "$srcdir" ]; then
     exit 2
 fi
 
-RSYNC="$rsync_bin"
-#RSYNC="valgrind --tool=addrcheck $rsync_bin"
-
-export rsync_bin RSYNC setfacl_nodef
-
 skipped=0
 missing=0
 passed=0
@@ -195,7 +210,7 @@ failed=0
 # Prefix for scratch directory.  We create separate directories for
 # each test case, so that they can be left behind in case of failure
 # to aid investigation.
-scratchbase="`pwd`"/testtmp
+scratchbase="$TOOLDIR"/testtmp
 echo "    scratchbase=$scratchbase"
 
 suitedir="$srcdir/testsuite"
@@ -206,7 +221,7 @@ prep_scratch() {
     [ -d "$scratchdir" ] && rm -rf "$scratchdir"
     mkdir "$scratchdir"
     # Get rid of default ACLs and dir-setgid to avoid confusing some tests.
-    $setfacl_nodef "$scratchdir"
+    $setfacl_nodef "$scratchdir" || true
     chmod g-s "$scratchdir"
     return 0
 }
